/*
 * This file is part of PLaSK (https://plask.app) by Photonics Group at TUL
 * Copyright (c) 2022 Lodz University of Technology
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 */
#ifndef PLASK__AIR_H
#define PLASK__AIR_H

/** @file
This file contains air
*/

#include "material.hpp"

namespace plask { namespace materials {

/**
 * Represent air, its physical properties.
 */
struct PLASK_API Air: public Material {

    static constexpr const char* NAME = "air";

    std::string name() const override;
    Kind kind() const override;
    double lattC(double T, char x) const override;
    double Eg(double T, double e, char point) const override;
    double CB(double T, double e, char point) const override;
    double VB(double T, double e, char point, char hole) const override;
    double Dso(double T, double e) const override;
    double Mso(double T, double e) const override;
    Tensor2<double> Me(double T, double e, char point) const override;
    Tensor2<double> Mhh(double T, double e) const override;
    Tensor2<double> Mlh(double T, double e) const override;
    Tensor2<double> Mh(double T, double e) const override;
    double eps(double T) const override;
    double chi(double T, double e, char point) const override;
    double Ni(double T) const override;
    double Nf(double T) const override;
    double EactD(double T) const override;
    double EactA(double T) const override;
    Tensor2<double> mob(double T) const override;
    Tensor2<double> cond(double T) const override;
    ConductivityType condtype() const override;
    double A(double T) const override;
    double B(double T) const override;
    double C(double T) const override;
    double D(double T) const override;
    Tensor2<double> thermk(double T, double h) const override;
    double dens(double T) const override;
    double cp(double T) const override;
    double nr(double lam, double T, double n = .0) const override;
    double absp(double lam, double T) const override;
    Tensor2<double> mobe(double T) const override;
    Tensor2<double> mobh(double T) const override;
    double Na() const override;
    double Nd() const override;
  protected:
    bool isEqual(const Material& other) const override;
};


}} // namespace plask::materials

#endif	//PLASK__AIR_H
